@@ -9,177 +9,130 @@ pub(crate) mod postfix;
 pub(crate) mod interpret;
 pub(crate) mod operator;
 
-#[derive(Debug, Clone, Eq, PartialEq)]
-pub(crate) struct InputPos {
-    line: usize,
-    ch: usize,
-}
-
-impl InputPos {
-    pub(crate) fn new(line: usize, ch: usize) -> Self {
-        Self {
-            line, ch
-        }
-    }
-
-    pub(crate) fn next(&mut self) {
-        self.ch += 1
-    }
-
-    pub(crate) fn newline(&mut self) {
-        self.line += 1;
-        self.ch = 0;
-    }
-}
-
-impl Default for InputPos {
-    fn default() -> Self {
-        Self::new(1, 1)
-    }
-}
-
-impl Display for InputPos {
-    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
-        write!(f, "{}:{}", self.line, self.ch)
-    }
-}
-
 /// An enum representing an error that occurred
-/// This is used by the Error struct to represent errors
 /// This allows for user handling of errors while still allowing them to just be
 /// printed out if custom handling of errors is not needed.
 #[derive(Debug, Clone, Eq, PartialEq)]
-pub enum ErrorType {
-    /// An error in which the input attempts a division by zero, which is undefined
+pub enum Error {
+    /// An error in which the input attempts a division by zero, which is undefined.
     DivByZero,
-    /// An error in which the input attempts to raise a number to a negative power, which is undefined
+    /// An error in which the input attempts to raise a number to a negative power, which is undefined.
     NegativeExponent,
-    /// An error in which the input contains an invalid character that can not be parsed
-    /// c is the invalid character
-    InvalidCharacter { c: char },
-    /// An error in which the input contains an invalid number (i.e. 2 decimal points)
-    /// found is the number in string form (it can't be parsed)
-    InvalidNumber { found: String },
-    /// An error in which the parser expected something but got something else (invalid input)
-    /// expected is the expected input
-    /// found is what was found
-    Expected { expected: String, found: String },
-    /// when the parser reaches an unexpected End Of Input when it is still expecting more input
+    /// An error in which the input contains an invalid character that can not be parsed.
+    InvalidCharacter {
+        /// The invalid character
+        c: char
+    },
+    /// An error in which the input contains an invalid number (i.e. 2 decimal points).
+    InvalidNumber {
+        /// The number in string form that couldn't be parsed
+        found: String
+    },
+    /// An error in which the parser expected something but got something else (invalid input).
+    Expected {
+        ///The expected input
+        expected: String,
+        /// What was found
+        found: String
+    },
+    /// when the parser reaches an unexpected End Of Input when it is still expecting more input.
     UnexpectedEOI,
-    /// When the interpreter finds an invalid operand, such as a variable identifier when none are defined
-    /// op: the invalid operand
-    InvalidOperand { op: String },
-    /// When the interpreter finds an invalid operator, such as a '=' while solving
-    /// op: the invalid operator
-    InvalidOperator { op: String },
-    /// When the interpreter has an invalid output, such as multiple values or a remaining operator
-    /// reason: the error message
-    InvalidExpression { reason: String },
-    /// When interpreting with definitions finds an undefined variable
-    /// name: the name of the undefined variable
-    UndefinedVariable { name: String },
-    /// When interpreting with definitions finds an undefined function
-    /// name: the name of the undefined function
-    UndefinedFunction { name: String },
-    /// When a function is called with the wrong number of arguments
-    /// name: the name of the function
-    /// expected: the number of arguments expected
-    /// got: the number of arguments received
-    InvalidArgumentCount { name: String, expected: usize, got: usize },
-    /// When something other than a number or variable is passed to a function
-    /// name: the name of the function
-    /// value: the value that was passed
-    InvalidArgument { name: String, value: String },
-    /// When the interpreter finds an operator after another operator that should not be there
-    /// op: the operator that was found
-    InvalidLeadingOperator { op: String },
-    /// When the interpreter expects an operator (i.e. after a number) but gets something else
+    /// When the interpreter finds an invalid operand, such as a variable identifier when none are defined.
+    InvalidOperand {
+        /// The invalid operand
+        op: String
+    },
+    /// When the interpreter finds an invalid operator, such as a '=' while solving.
+    InvalidOperator {
+        /// The invalid operator
+        op: String
+    },
+    /// When the interpreter has an invalid output, such as multiple values or a remaining operator.
+    InvalidExpression {
+        /// The reason for the error
+        reason: String
+    },
+    /// When interpreting with definitions finds an undefined variable.
+    UndefinedVariable {
+        /// The name of the undefined variable
+        name: String
+    },
+    /// When interpreting with definitions finds an undefined function.
+    UndefinedFunction {
+        /// The name of the undefined function
+        name: String
+    },
+    /// When a function is called with the wrong number of arguments.
+    InvalidArgumentCount {
+        /// The name of the function
+        name: String,
+        /// The expected number of arguments
+        expected: usize,
+        /// The number of arguments received
+        got: usize
+    },
+    /// When something other than a number or variable is passed to a function.
+    InvalidArgument {
+        /// The name of the function
+        name: String,
+        /// The invalid value that was passed
+        value: String
+    },
+    /// When the interpreter finds an operator after another operator that should not be there.
+    InvalidLeadingOperator {
+        /// the operator that was found
+        op: String
+    },
+    /// When the interpreter expects an operator (i.e. after a number) but gets something else.
     MissingOperator,
-    /// When there is an incomplete pair of parentheses (i.e. open with no close or vice versa)
-    /// found: the parenthesis that was found (either '(' or ')')
-    /// missing: the parenthesis that was expected (either ')' or '(')
-    MismatchedParentheses { found: char, missing: char },
-    /// Custom error messages
-    /// contains a String of the error message
+    /// When there is an incomplete pair of parentheses (i.e. open with no close or vice versa).
+    MismatchedParentheses {
+        /// The parenthesis that was found ('(' or ')')
+        found: char,
+        /// The parenthesis that was missing ('(' or ')')
+        missing: char
+    },
+    /// Custom error messages.
+    /// contains a String of the error message.
     /// this is not used by this program and is only used for custom error messages by the user
-    ///
     Other(String),
 }
 
-impl Display for ErrorType {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        match self {
-            ErrorType::DivByZero => write!(f, "Can't divide by zero"),
-            ErrorType::NegativeExponent => write!(f, "Can't raise a value to a negative power"),
-            ErrorType::InvalidCharacter { c } => write!(f, "Invalid character: {}", c),
-            ErrorType::InvalidNumber { found } => write!(f, "Invalid number: {}", found),
-            ErrorType::Expected { expected, found } => write!(f, "Expected '{}', found '{}'", expected, found),
-            ErrorType::UnexpectedEOI => write!(f, "Unexpected end of input"),
-            ErrorType::InvalidOperand { op } => write!(f, "Invalid operand: {}", op),
-            ErrorType::InvalidOperator { op } => write!(f, "Invalid operator: {}", op),
-            ErrorType::InvalidExpression { reason } => write!(f, "Invalid expression: {}", reason),
-            ErrorType::UndefinedVariable { name } => write!(f, "Undefined variable: {}", name),
-            ErrorType::UndefinedFunction { name } => write!(f, "Undefined function: {}", name),
-            ErrorType::InvalidArgumentCount { name, expected, got } => write!(f, "Invalid argument count for function '{}': expected {}, got {}", name, expected, got),
-            ErrorType::InvalidArgument { name, value } => write!(f, "Invalid argument for function '{}': {}", name, value),
-            ErrorType::InvalidLeadingOperator { op } => write!(f, "Invalid leading operator: {}", op),
-            ErrorType::MissingOperator => write!(f, "Missing operator"),
-            ErrorType::MismatchedParentheses { found, missing } => write!(f, "Mismatched parentheses: found '{}', missing '{}'", found, missing),
-            ErrorType::Other(s) => write!(f, "{}", s),
-        }
-    }
-}
-
-/// Stores errors from the parser if any occur.
-#[derive(Debug, Clone, Eq, PartialEq)]
-pub struct Error {
-    error: ErrorType,
-    pos: Option<InputPos>,
-}
-
 impl Error {
-    pub(crate) fn new(etype: ErrorType, pos: InputPos) -> Self {
-        Self {
-            error: etype,
-            pos: Some(pos),
+    pub fn arg_count<S: Into<String>>(name: S, expected: usize, got: usize) -> Error {
+        Error::InvalidArgumentCount {
+            name: name.into(),
+            expected,
+            got
         }
     }
-
-    pub(crate) fn new_gen(etype: ErrorType) -> Self {
-        Self {
-            error: etype,
-            pos: None,
-        }
-    }
-
-    /// a way for users to create new errors
-    pub fn create(etype: ErrorType) -> Self {
-        Self::new_gen(etype)
-    }
-
-    /// Returns the error message.
-    pub fn get_error(&self) -> &ErrorType {
-        &self.error
-    }
-
-    /// returns the location of the error as (line, character)
-    /// this struct uses a private `InputPos` struct to store the position,
-    /// but that is not exposed to the user.
-    pub fn get_loc(&self) -> (usize, usize) {
-        (self.pos.as_ref().unwrap().line, self.pos.as_ref().unwrap().ch)
-    }
 }
 
 impl Display for Error {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        if self.pos.is_some() {
-            write!(f, "Error: {} at {}", self.error, self.pos.as_ref().unwrap())
-        } else {
-            write!(f, "Error: {}", self.error)
+        match self {
+            Error::DivByZero => write!(f, "Can't divide by zero"),
+            Error::NegativeExponent => write!(f, "Can't raise a value to a negative power"),
+            Error::InvalidCharacter { c } => write!(f, "Invalid character: {}", c),
+            Error::InvalidNumber { found } => write!(f, "Invalid number: {}", found),
+            Error::Expected { expected, found } => write!(f, "Expected '{}', found '{}'", expected, found),
+            Error::UnexpectedEOI => write!(f, "Unexpected end of input"),
+            Error::InvalidOperand { op } => write!(f, "Invalid operand: {}", op),
+            Error::InvalidOperator { op } => write!(f, "Invalid operator: {}", op),
+            Error::InvalidExpression { reason } => write!(f, "Invalid expression: {}", reason),
+            Error::UndefinedVariable { name } => write!(f, "Undefined variable: {}", name),
+            Error::UndefinedFunction { name } => write!(f, "Undefined function: {}", name),
+            Error::InvalidArgumentCount { name, expected, got } => write!(f, "Invalid argument count for function '{}': expected {}, got {}", name, expected, got),
+            Error::InvalidArgument { name, value } => write!(f, "Invalid argument for function '{}': {}", name, value),
+            Error::InvalidLeadingOperator { op } => write!(f, "Invalid leading operator: {}", op),
+            Error::MissingOperator => write!(f, "Missing operator"),
+            Error::MismatchedParentheses { found, missing } => write!(f, "Mismatched parentheses: found '{}', missing '{}'", found, missing),
+            Error::Other(s) => write!(f, "{}", s),
         }
     }
 }
 
+/// The type used for operations.
 #[derive(Debug, Clone)]
 pub struct Number {
     value: f64,
@@ -286,35 +239,35 @@ impl Default for Functions<'_> {
         let mut funcs = Functions::new();
         funcs.register("log", |args| {
             if args.len() != 2 {
-                return Err(Error::create(ErrorType::InvalidArgumentCount { name: "log".to_string(), expected: 2, got: args.len() }));
+                return Err(Error::arg_count("log", 2, args.len()));
             }
             Ok(Number::new(args[1].as_f64().log(args[0].as_f64())))
         });
 
         funcs.register("sqrt", |args| {
             if args.len() != 1 {
-                return Err(Error::create(ErrorType::InvalidArgumentCount { name: "sqrt".to_string(), expected: 1, got: args.len() }));
+                return Err(Error::arg_count("sqrt", 1, args.len()));
             }
             Ok(Number::new(args[0].as_f64().sqrt()))
         });
 
         funcs.register("sin", |args| {
             if args.len() != 1 {
-                return Err(Error::create(ErrorType::InvalidArgumentCount { name: "sin".to_string(), expected: 1, got: args.len() }));
+                return Err(Error::arg_count("sin", 1, args.len()));
             }
             Ok(Number::new(args[0].as_f64().sin()))
         });
 
         funcs.register("cos", |args| {
             if args.len() != 1 {
-                return Err(Error::create(ErrorType::InvalidArgumentCount { name: "cos".to_string(), expected: 1, got: args.len() }));
+                return Err(Error::arg_count("cos", 1, args.len()));
             }
             Ok(Number::new(args[0].as_f64().cos()))
         });
 
         funcs.register("tan", |args| {
             if args.len() != 1 {
-                return Err(Error::create(ErrorType::InvalidArgumentCount { name: "tan".to_string(), expected: 1, got: args.len() }));
+                return Err(Error::arg_count("tan", 1, args.len()));
             }
             Ok(Number::new(args[0].as_f64().tan()))
         });
@@ -329,13 +282,12 @@ impl Default for Functions<'_> {
 /// ```
 /// use calc_lib::solve;
 ///
-/// let x = solve("(1 + 2) * 3");
-/// if x.is_err() {
+/// let solved = solve("(1 + 2) * 3");
+/// if solved.is_err() {
 ///     panic!("{}", x.unwrap_err());
 /// }
-/// assert_eq!(x.unwrap().as_i128(), 9);
+/// assert_eq!(solved.unwrap().as_i128(), 9);
 /// ```
-///
 pub fn solve<S: Into<String>>(input: S) -> Result<Number, Error> {
     let mut input = InputReader::new(input.into());
     let mut tokens = lex::lex(&mut input, false)?;
@@ -361,16 +313,19 @@ pub fn solve<S: Into<String>>(input: S) -> Result<Number, Error> {
 ///
 /// # Usage with functions:
 /// ```
-/// use calc_lib::{Definitions, Functions, Number, Error, solve_defs, ErrorType};
+/// use calc_lib::{Definitions, Functions, Number, solve_defs, Error};
 ///
 /// let mut defs = Definitions::new();
 /// defs.register("x", Number::new(3));
 ///
 /// let mut funcs = Functions::new();
-/// // this shows the definition of the log function, which is already implemented in `Functions::default();`
+/// // this shows the definition of the log function,
+/// // which is already implemented in `Functions::default();`
 /// funcs.register("log", |args| {
+///     // this function takes two arguments, base and the number
 ///     if args.len() != 2 {
-///         return Err(Error::create(ErrorType::InvalidArgumentCount { name: "log".to_string(), expected: 2, got: args.len() }));
+///         // if the number of arguments is not 2, return an error
+///         return Err(Error::arg_count("log", 2, args.len()));
 ///     }
 ///     Ok(Number::new(args[1].as_f64().log(args[0].as_f64())))
 ///  });
@@ -378,7 +333,6 @@ pub fn solve<S: Into<String>>(input: S) -> Result<Number, Error> {
 /// let solved = solve_defs("log(2, 16)", Some(&defs), Some(&funcs));
 /// assert_eq!(solved.unwrap().as_i128(), 4);
 /// ```
-///
 pub fn solve_defs<S: Into<String>>(input: S, definitions: Option<&Definitions>, functions: Option<&Functions>) -> Result<Number, Error> {
     let mut input = InputReader::new(input.into());
     let mut tokens = lex::lex(&mut input, true)?;
@@ -428,4 +382,18 @@ mod test {
         }
         assert_eq!(solved5.unwrap().as_f64(), 4.0);
     }
+
+    #[test]
+    fn test_specific() {
+        let exprs = "2 ^ (2 * 2)";
+        let solved_res = solve(exprs);
+        if solved_res.is_err() {
+            panic!("{}", solved_res.err().unwrap());
+        }
+        let solved = solved_res.unwrap();
+        assert_eq!(solved.clone().as_i128(), 16);
+
+        let mut defs = Definitions::new();
+        defs.register("solved", Number::new(solved.as_f64()));
+    }
 }
\ No newline at end of file
